#!/bin/bash -l

declare -A DATAFILES=(["uniformgridcpu"]="cpu_benchmark.sqlite3" ["uniformgridgpu"]="gpu_benchmark_*.sqlite3")
declare -A PROCESSINGTYPES=(["uniformgridcpu"]="uniformgrid_sqlite" ["uniformgridgpu"]="uniformgrid_sqlite")
HAS_CUDA_GPU="genoa2 medusa"
HAS_AMD_GPU="rome2 euryale"
HOST="$1"

if [ -z "${BENCHMARKS}" ];
then
    BENCHMARKS=$(ls jobscript/benchmarks | xargs basename -s .sh)
fi
BENCHMARKS_TO_BUILD='apps/benchmarks/UniformGridCPU apps/showcases/FreeSurface'

USE_CUDA="OFF"
if [[ ${HAS_CUDA_GPU} == *${HOST}* ]];
then
    USE_CUDA="ON"
    BENCHMARKS_TO_BUILD+=" apps/benchmarks/UniformGridGPU"
fi

USE_HIP="OFF"
if [[ ${HAS_HIP_GPU} == *${HOST}* ]];
then
    USE_HIP="ON"
    BENCHMARKS_TO_BUILD+=" apps/benchmarks/UniformGridGPU"
fi

cat << EOF
include:
  - "ci/base_jobs.yml"

stages:
  - build
  - benchmark
  - final

variables:
  WALBERLA_GITLAB_INSTANCE: "${WALBERLA_GITLAB_INSTANCE}"
  WALBERLA_PROJECT_ID: "${WALBERLA_PROJECT_ID}"
  WALBERLA_BRANCH: "${WALBERLA_BRANCH}"
  WALBERLA_COMMIT: "${WALBERLA_COMMIT}"

.export_variables:
  before_script:
    - export WALBERLA_GITLAB_INSTANCE="${WALBERLA_GITLAB_INSTANCE}"
    - export WALBERLA_PROJECT_ID="${WALBERLA_PROJECT_ID}"
    - export WALBERLA_BRANCH="${WALBERLA_BRANCH}"
    - export WALBERLA_COMMIT="${WALBERLA_COMMIT}"

build_${HOST}:
  stage: build
  extends:
    - .submit_job
    - .seralize
  variables:
    HOST: "${HOST}"
    SCRIPT: "jobscript/build.sh"
    USE_MPI: "ON"
    USE_OMP: "OFF"
    USE_CUDA: "${USE_CUDA}"
    USE_HIP: "${USE_HIP}"
    BENCHMARKS_TO_BUILD: "${BENCHMARKS_TO_BUILD}"
  before_script:
    - !reference [.export_variables, before_script]
EOF

for BENCHMARK in $BENCHMARKS; do
    if [[ "${BENCHMARK}" == "viscousldc" ]] && [[ "${WALBERLA_BRANCH}" != "viscLDCwithFSLBM" ]]; then
        continue
    fi
    if [[ "${BENCHMARK}" == *"gpu"* ]] && [[ "${USE_CUDA}" == "OFF" ]]; then
        continue
    fi
    job_name="benchmark_${BENCHMARK}_${HOST}"
    cat << EOF

${job_name}:
  stage: benchmark
  extends:
    - .submit_job
    - .seralize
  when: always
  variables:
    HOST: "${HOST}"
    SCRIPT: "jobscript/benchmarks/${BENCHMARK}.sh"
    TYPE: "${PROCESSINGTYPES[$BENCHMARK]}"
    DATAFILE: "${DATAFILES[$BENCHMARK]}"
    USE_MPI: "ON"
    USE_OMP: "OFF"
    USE_CUDA: "${USE_CUDA}"
    USE_HIP: "${USE_HIP}"
    PERF_GROUP: "MEM_DP"
    LOG_LEVEL: "info"
    VISABLE_DEVICE: "0"
  before_script:
    - !reference [.export_variables, before_script]
EOF
    if [ "${BENCHMARK}" != "viscousldc" ]; then
        cat << EOF
  after_script:
    - !reference [.export_variables, before_script]
    - !reference [.upload, script]
EOF
    fi
    if [[ "${BENCHMARK}" == *"uniformgridgpu"* ]]; then
        cat << EOF
    - export DATAFILE=$(echo gpu_profile_*.sqlite3)
    - !reference [.upload, script]
EOF
    fi
done

cat << EOF

upload_pipeline_stats:
  stage: final
  extends:
    - .seralize
  tags:
    - testcluster
  variables:
    NO_SLURM_SUBMIT: 1
  when: always
  script:
    - !reference [.load_cbutil, before_script]
    - python3 upload_job_infos.py
    - !reference [.load_cbutil, after_script]
EOF
