cd "${BASE_DIR}" || exit 1

# Get Sources
if [ ! -d "${FULL_PATH}" ];
then
    echo "Cloning into ${FULL_PATH}"
    git clone -b ${WALBERLA_BRANCH} ${WALBERLA_URL} ${FULL_PATH}
fi

cd "${FULL_PATH}" || exit 1
git fetch -a
git checkout $WALBERLA_BRANCH
[[ -n "$WALBERLA_COMMIT" ]] && git checkout $WALBERLA_COMMIT
git pull

# PYTHON Setup
if [ ! -d "${PYTHON_VENV}" ];
then
    python3 -m venv $PYTHON_VENV
fi
source $PYTHON_VENV/bin/activate
pip install --upgrade pip setuptools wheel jinja2 py-cpuinfo lbmpy pystencils
pip list

mkdir -p ${BUILD_DIR}
cd ${BUILD_DIR}

# cmake setup

export NUM_CORES=$(nproc --all)
export MAX_BUILD_CORES=$(( $(awk '( $1 == "MemTotal:" ) { print $2 }' /proc/meminfo) / ( 4 * 1024 * 1024  ) ))
[[ $MAX_BUILD_CORES -lt $NUM_CORES ]] && export NUM_BUILD_CORES=$MAX_BUILD_CORES || export NUM_BUILD_CORES=$NUM_CORES

CMAKE_BUILD_TYPE="Release"
WALBERLA_BUFFER_DEBUG="OFF"
WALBERLA_DOUBLE_ACCURACY="ON"
WALBERLA_ENABLE_GUI="OFF"
WALBERLA_BUILD_WITH_METIS="OFF"
WALBERLA_BUILD_WITH_PARMETIS="OFF"
WALBERLA_BUILD_WITH_PYTHON="ON"
WALBERLA_BUILD_WITH_CODEGEN="ON"
WALBERLA_STL_BOUNDS_CHECKS="OFF"
WALBERLA_OPTIMIZE_FOR_LOCALHOST="ON"
WALBERLA_BUILD_WITH_FASTMATH="ON"
WALBERLA_BUILD_WITH_LTO="ON"
WALBERLA_BUILD_WITH_LIKWID_MARKERS="ON"
WALBERLA_BUILD_WITH_MPI="${USE_MPI}"
WALBERLA_BUILD_WITH_CUDA="${USE_CUDA}"
WALBERLA_BUILD_WITH_HIP="${USE_HIP}"
WALBERLA_BUILD_WITH_OPENMP=${USE_OMP}

echo "Building walberla in $(pwd)"

cmake ${FULL_PATH} \
    -DWALBERLA_BUFFER_DEBUG=$WALBERLA_BUFFER_DEBUG \
    -DWALBERLA_BUILD_TESTS=ON \
    -DWALBERLA_BUILD_BENCHMARKS=ON \
    -DWALBERLA_BUILD_TUTORIALS=OFF \
    -DWARNING_ERROR=ON \
    -DWALBERLA_BUILD_WITH_MPI=$WALBERLA_BUILD_WITH_MPI \
    -DWALBERLA_BUILD_WITH_CUDA=$WALBERLA_BUILD_WITH_CUDA \
    -DWALBERLA_BUILD_WITH_HIP=$WALBERLA_BUILD_WITH_HIP \
    -DWALBERLA_BUILD_WITH_PYTHON=$WALBERLA_BUILD_WITH_PYTHON \
    -DWALBERLA_BUILD_WITH_OPENMP=$WALBERLA_BUILD_WITH_OPENMP \
    -DWALBERLA_BUILD_WITH_LIKWID_MARKERS=$WALBERLA_BUILD_WITH_LIKWID_MARKERS \
    -DCMAKE_BUILD_TYPE=$CMAKE_BUILD_TYPE \
    -DMPIEXEC_PREFLAGS=$MPIEXEC_PREFLAGS \
    -DWALBERLA_DOUBLE_ACCURACY=$WALBERLA_DOUBLE_ACCURACY \
    -DWALBERLA_BUILD_WITH_METIS=$WALBERLA_BUILD_WITH_METIS \
    -DWALBERLA_BUILD_WITH_PARMETIS=$WALBERLA_BUILD_WITH_PARMETIS \
    -DWALBERLA_ENABLE_GUI=$WALBERLA_ENABLE_GUI \
    -DWALBERLA_BUILD_WITH_CODEGEN=$WALBERLA_BUILD_WITH_CODEGEN \
    -DWALBERLA_STL_BOUNDS_CHECKS=$WALBERLA_STL_BOUNDS_CHECKS \
    -DWALBERLA_OPTIMIZE_FOR_LOCALHOST=$WALBERLA_OPTIMIZE_FOR_LOCALHOST \
    -DWALBERLA_BUILD_WITH_FASTMATH=$WALBERLA_BUILD_WITH_FASTMATH \
    -DWALBERLA_BUILD_WITH_LTO=$WALBERLA_BUILD_WITH_LTO

cmake . -LA

DIRS_TO_BUILD="tests/core ${BENCHMARKS_TO_BUILD}"
# actually build benchmarks
for dir_to_build in ${DIRS_TO_BUILD}; do
    [ -d $dir_to_build ] || ( echo "${dir_to_build} not available on current branch!" && continue )
    make -C "$dir_to_build" -j $NUM_BUILD_CORES || echo "Build fails for ${dir_to_build}"
done
if [ "${WALBERLA_BRANCH}" = "viscLDCwithFSLBM" ];
then
    make -C "apps/showcases/FreeSurface" -j $NUM_BUILD_CORES ViscousLDCCodegen
fi

# runing the core tests to ensure that everything is working
cd tests/core || exit 1
ctest -C $CMAKE_BUILD_TYPE --output-on-failure -j $NUM_CORES -T Test -R ReduceTest
